package de.hdm_stuttgart.battlearena.Model.DataStorage.Classes;

import de.hdm_stuttgart.battlearena.Controller.Enum.GameMode;
import de.hdm_stuttgart.battlearena.Controller.Enum.GameState;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.ENUMs.ThreadStatus;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Exceptions.ParserException;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.HashGenerator;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.Utilities.Parser;
import de.hdm_stuttgart.battlearena.Model.Entity.EntityClass;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class RuntimeInfo {

    private static final Logger log = LogManager.getLogger(RuntimeInfo.class);
    private static final RuntimeInfo runtimeInfoSingleton = new RuntimeInfo();
    private String mapDataGame;
    private MapData mapGame;
    private boolean offlineMode;  //if Account-Type is online but no SQL connection: start game without stats tracking
    private GameState gameState = GameState.MENU; //Default value: MENU
    private GameMode gameMode = GameMode.LOCAL; //Default value: LOCAL

    //Default value: HUMAN
    private EntityClass playerOneClass = EntityClass.HUMAN;
    private EntityClass playerTwoClass = EntityClass.HUMAN;

    //Stats Tracking during Gameplay
    private int kills;
    private int deaths;
    private int blocksDestroyed;
    private int gameTime;
    private boolean gameWon;






    private RuntimeInfo(){};

    public static RuntimeInfo getInstance(){
        return runtimeInfoSingleton;
    }

    public MapData generateMap(String mapName, String mapData) throws ParserException {
        try {
            Parser.mapDataValid(mapData);
            Parser.mapNameValid(mapName);
            return new MapData(HashGenerator.hashAndHex(mapData), mapName, 18, 18, mapData);
        }
        catch(Exception e){
            log.error(e);
            throw new ParserException(e.getMessage());
        }
    }

    public void setGameMap(MapData mapSelected){    //Create Game Scene Dropdown list directly gives reference to selected map
        try {
            mapGame = mapSelected;
            mapDataGame = mapSelected.getMapData();
        }
        catch(Exception e){
            log.error(e);
        }
    }

    public GameState getGameState() {
        return gameState;
    }

    public void setGameState(GameState gameState) {
        this.gameState = gameState;
    }

    public GameMode getGameMode() {
        return gameMode;
    }

    public void setGameMode(GameMode gameMode) {
        this.gameMode = gameMode;
    }

    public EntityClass getPlayerOneClass() {
        return playerOneClass;
    }

    public void setPlayerOneClass(EntityClass playerOneClass) {
        this.playerOneClass = playerOneClass;
    }

    public EntityClass getPlayerTwoClass() {
        return playerTwoClass;
    }

    public void setPlayerTwoClass(EntityClass playerTwoClass) {
        this.playerTwoClass = playerTwoClass;
    }

    public String getMapDataGame() {
        return mapDataGame;
    }

    public int getKills() {
        return kills;
    }

    public void setKills(int kills) {
        this.kills = kills;
    }

    public void addKill() {
        kills = kills + 1;
    }

    public int getDeaths() {
        return deaths;
    }

    public void setDeaths(int deaths) {
        this.deaths = deaths;
    }

    public void addDeath() {
        deaths = deaths + 1;
    }

    public int getBlocksDestroyed() {
        return blocksDestroyed;
    }

    public void setBlocksDestroyed(int blocksDestroyed) {
        this.blocksDestroyed = blocksDestroyed;
    }

    public void addBlockDestroyed() {
        blocksDestroyed = blocksDestroyed + 1;
    }

    public int getGameTime() {
        return gameTime;
    }

    public void setGameTime(int gameTime) {
        this.gameTime = gameTime;
    }

    public void addGameTime() {
        gameTime = gameTime + 1;
    }

    public boolean isGameWon() {
        return gameWon;
    }

    public void setGameWon(boolean gameWon) {
        this.gameWon = gameWon;
    }

}
